-----------------------
--- Module Creation ---
-----------------------
local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("questsGivers", "AceEvent-3.0")

-----------------------
--- Library loading ---
-----------------------
local LQG = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-QuestsGivers")
local LDQ = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-DailyQuests")
local LFQ = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-ForbidenQuests")

--------------------------------------------
--- Copy global functions to local scope ---
--------------------------------------------
local select = select
local format = string.format
local GetGossipAvailableQuests, GetGossipActiveQuests = GetGossipAvailableQuests, GetGossipActiveQuests
local GetNumActiveQuests, GetNumAvailableQuests = GetNumActiveQuests, GetNumAvailableQuests
local GetActiveTitle, GetActiveLevel = GetActiveTitle, GetActiveLevel
local GetAvailableTitle, GetAvailableLevel = GetAvailableTitle, GetAvailableLevel

-----------------
--- variables ---
-----------------
module.modName = LQG["Quests givers"]
local prdb, options
local automaticAcceptChoices = {[1] = LQG["All quests"], [2] = LQG["Only daily quests"], [3] = LQG["All but daily quests"], [4] = LQG["Never"]}
local dailyQuests = {}
for k, v in pairs(LDQ) do
	dailyQuests[v] = true
end

-----------------------
--- Default profile ---
-----------------------
local defaults = {
	profile = {
		addQuestLevel = true,
		fadeNotCompletedQuests = true,
		automaticAccept = 1,
		automaticAcceptEscort = true,
		automaticTurnIn = true,
	}
}

---------------
--- Options ---
---------------
local optGetter, optSetter
do
	function optGetter(info)
		return prdb[info[#info]]
	end
	function optSetter(info, value)
		prdb[info[#info]] = value
	end
end

local function getOptions()
	if not options then
		options = {
			type = "group",
			childGroups = "tab",
			name = module.modName,
			get = optGetter,
			set = optSetter,
			args = {
				addQuestLevel = {
					name = LQG["Add quest level to the quest name"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				fadeNotCompletedQuests = {
					name = LQG["Fade quest name that are not complete"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				automaticAccept = {
					name = LQG["Accept quests automatically"],
					type = "select",
					values = automaticAcceptChoices,
					width = "full",
					order = 3,
				},
				automaticAcceptEscort = {
					name = LQG["Accept escort quest started by other player automatically"],
					type = "toggle",
					width = "full",
					order = 4,
				},
				automaticTurnIn = {
					name = LQG["Automatic quest turn-in"],
					type = "toggle",
					width = "full",
					order = 5,
				},
			},
		}
	end
	return options
end



function module:OnInitialize()
	self.db = core.db:RegisterNamespace("questsGivers", defaults, "Default")
	prdb = self.db.profile
	self:SetEnabledState(core:getModuleEnabled("questsGivers"))
	core:registerModuleOptions("questsGivers", getOptions, module.modName)
end

function module:OnEnable()
	self:RegisterEvent("GOSSIP_SHOW")
	self:RegisterEvent("QUEST_GREETING")
	self:RegisterEvent("QUEST_COMPLETE")
	self:RegisterEvent("QUEST_DETAIL")
	self:RegisterEvent("QUEST_PROGRESS")
	self:RegisterEvent("QUEST_ACCEPT_CONFIRM")
end

local function questComplete(uid)
	if not uid then return false end
	return core.quests[uid].complete[core.playerName] == 1 or core.quests[uid].numObjectives == 0
end

local function modifyQuestButton(button, title, level)
	if prdb.addQuestLevel and title then
		button:SetText(format("[%s] %s", level == -1 and '*' or level, title))
	end
end

local function fadeQuestButton(button, fade, complete)
	if prdb.fadeNotCompletedQuests and fade and not complete then
		button:SetVertexColor(0.5, 0.5, 0.5, 0.5)
	else
		button:SetVertexColor(1, 1, 1, 1)
	end
end

local function clickQuest(button)
	if button:IsVisible() then
		local title = button:GetText()
		title = title:gsub("%(.+)%]", "")
		title = title:gsub("|c%x%x%x%x%x%x%x%x(.+)|r","%1")
		title = title:gsub("(.+) %((.+)%)", "%1")
		if button.type == "Available" and (prdb.automaticAccept == 1 or (prdb.automaticAccept == 2 and dailyQuests[title]) or (prdb.automaticAccept == 3 and not dailyQuests[title])) then
			button:Click()
		elseif button.type == "Active" and prdb.automaticTurnIn then
			if (core.questsTitleToUid[title] and questComplete(core.questsTitleToUid[title])) or (not core.questsTitleToUid[title]) then
				button:Click()
			end
		end
	end
end

local function gossipLoop(buttonIndex, fade, ...)
	local numQuests = select('#', ...)
	for i=2, numQuests, 3 do
		local button = _G["GossipTitleButton"..buttonIndex]
		local level = select(i, ...)
		local title = button:GetText()
		modifyQuestButton(button, title, level)
		fadeQuestButton(_G["GossipTitleButton"..buttonIndex.."GossipIcon"], fade, questComplete(core.questsTitleToUid[select(i-1, ...)]))
		buttonIndex = buttonIndex + 1
	end
	return buttonIndex + 1
end

function module:GOSSIP_SHOW()
	local buttonIndex = 1
	if GetGossipAvailableQuests() then
		buttonIndex = gossipLoop(buttonIndex, false, GetGossipAvailableQuests())
	end
	if GetGossipActiveQuests() then
		buttonIndex = gossipLoop(buttonIndex, true, GetGossipActiveQuests())
	end
	if IsShiftKeyDown() then return end
	for i = 1, 32 do
		clickQuest(_G["GossipTitleButton" .. i])
	end
end

function module:QUEST_GREETING()
	local numActive, numAvailable = GetNumActiveQuests(), GetNumAvailableQuests()
	for i = 1, numActive do
		local title, level = GetActiveTitle(i), GetActiveLevel(i)
		local buttonName = "QuestTitleButton"..i
		local button = _G[buttonName]
		modifyQuestButton(button, title, level)
		fadeQuestButton(_G[buttonName.."QuestIcon"], true, questComplete(core.questsTitleToUid[title]))
	end
	for i = 1, numAvailable do
		local title, level = GetAvailableTitle(i), GetAvailableLevel(i)
		local index = i + numActive
		local buttonName = "QuestTitleButton"..index
		modifyQuestButton(_G[buttonName], title, level)
		fadeQuestButton(_G[buttonName.."QuestIcon"], true, questComplete(core.questsTitleToUid[title]))
	end
	if IsShiftKeyDown() then return end
	for i = 1, 32 do
		clickQuest(_G['QuestTitleButton' .. i])
	end
end

function module:QUEST_DETAIL()
	if not IsShiftKeyDown() and (prdb.automaticAccept == 1 or (prdb.automaticAccept == 2 and dailyQuests[title]) or (prdb.automaticAccept == 3 and not dailyQuests[title])) then
		AcceptQuest()
	end
end

function module:QUEST_COMPLETE(event)
	if not IsShiftKeyDown() and prdb.automaticTurnIn and GetNumQuestChoices() <= 1 then
		GetQuestReward(QuestFrameRewardPanel.itemChoice)
	end
end

function module:QUEST_PROGRESS()
	if not IsShiftKeyDown() and prdb.automaticTurnIn and IsQuestCompletable() then
		CompleteQuest()
	end
end

function module:QUEST_ACCEPT_CONFIRM()
	if not IsShiftKeyDown() and prdb.automaticAcceptEscort then
		ConfirmAcceptQuest()
	end
end